@@ -1,8 +1,8 @@
 #!/usr/bin/perl
 
-my $version = "2.15";
+my $version = "2.21";
 
-# Copyright (c) 2000 - 2002 Marco van Berkum
+# Copyright (c) 2000 - 2003 Marco van Berkum
 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -36,9 +36,30 @@ use Getopt::Std;
 
 # core functionality
 
-&getopts('AitfmsSdrVl:w:z:a:XF:M:O:P:');
+&getopts('AitfmsSdprVl:w:z:a:XF:M:O:P:g:');
 
-use vars qw($opt_A $opt_i $opt_t $opt_f $opt_m $opt_s $opt_S $opt_d $opt_r $opt_V $opt_l $opt_w $opt_z $opt_a $opt_X $opt_F $opt_O $opt_P);
+use vars qw(
+	$opt_A 
+	$opt_i 
+	$opt_t 
+	$opt_f 
+	$opt_m 
+	$opt_s 
+	$opt_S 
+	$opt_d 
+	$opt_p
+	$opt_r 
+	$opt_V 
+	$opt_l 
+	$opt_w 
+	$opt_z 
+	$opt_a 
+	$opt_X 
+	$opt_F 
+	$opt_O 
+	$opt_P 
+	$opt_g
+);
 
 if (!$opt_F) { $host = $ARGV[0]; } else {chomp $opt_F; $targetfile = $opt_F};
 
@@ -57,6 +78,7 @@ if (
         && !$opt_s
         && !$opt_S
         && !$opt_d
+	&& !$opt_p
         && !$opt_r
         && !$opt_w
         && !$opt_z
@@ -64,6 +86,7 @@ if (
         && !$opt_X
         && !$opt_F
         && !$opt_P 
+	&& !$opt_g
 	&& !$ARGV[1])
   )
 {
@@ -72,16 +95,18 @@ if (
         print("Available options:\n");
 	print("-O <output file>\n");
         print("-A\t\tAll scan types\n");
-	print("-i\t\tidentd scan\n");
+	print("-i\t\tIdentd scan\n");
 	print("-t\t\tSimple Telnet Fingerprint\n");
         print("-f\t\tFTP scan\n");
         print("-m\t\tMTA scan\n");
         print("-s\t\tSSH scan\n");
 	print("-S\t\tSpammer check\n");
         print("-d\t\tDNS scan\n");
+	print("-p\t\tPOP3 scan\n");
 	print("-r\t\tRegistrar whois lookup\n");
 	print("-V\t\tPrint version and exit\n");
 	print("-P <host:port>\tUse proxy for scanning\n");
+	print("-g <string>\tGoogle search \(uses strings from google.db when -A option is used\)\n");
         print("-l <type>\tloglevel\n");
 	print("\t\tc  critical (default)\n");
         print("\t\tv  verbose\n");
@@ -99,12 +124,15 @@ if (
 	print("-X\t\tX allow check\n\n");
         print("examples:\t./hackbot.pl -f -m -w oci localhost\n");
         print("\t\t./hackbot.pl -A 192.168.75.3 195.0.0.255\n");
-	print("\t\t./hackbot.pl -s 192.168.75.0\/26\n\n");
+	print("\t\t./hackbot.pl -s 192.168.75.0\/26\n");
+	print("\t\t./hackbot.pl -g britney www.geocities.com\n");
+	print("\t\t./hackbot.pl -A -lv localhost\n\n");
 	exit(0);
 }
 
 $database = "/usr/local/etc/cgi.db";
 $fingerprintdb = "/usr/local/etc/fingerprint.db";
+$googledb = "/usr/local/etc/google.db";
 
 # print(`clear`);
 print("\n");
@@ -112,7 +140,6 @@ print("\n");
 
 $noprint = 1; log_print(`date`,"c"); log_print("\n","c"); undef($noprint);
 
-       
 if($opt_l) {
         if(($opt_l !~ /^[cdv]+$/ ))
         {
@@ -244,6 +271,8 @@ sub scanit {
 	$opt_s="1";$opt_S="1";
         $opt_d="1";$opt_r="1";
 	$opt_w="a";$opt_X="1";
+	$opt_g="1";$opt_p="1";
+
 	#now you can undef opt_A in case of no web. 
 	# undef after this line
 	}
@@ -267,6 +296,7 @@ sub scanit {
         if($opt_m) {
 		header_reset();
                 &mta;
+		&evilsendmail;
         }
         if($opt_s) {
 		header_reset();
@@ -276,6 +306,10 @@ sub scanit {
 		header_reset();
                 &dns;
         }
+	if($opt_p) {
+		header_reset();
+		&pop3;
+	}
 	if($opt_r) {
 		header_reset();
 		&whois unless ($nowhois);
@@ -285,6 +319,19 @@ sub scanit {
 		header_reset();
 		&xcheck;
 	}
+        if($opt_g) {
+		header_reset();
+		$gdefined = $opt_g unless($opt_g == "1");
+		chomp($gdefined) unless(!$gdefined);
+      		my $google_handler = gethost("www.google.com");
+		        if(!$google_handler) {
+		                print("cannot resolve www.google.com, skipping google search\n");
+		                undef($opt_G);
+        		} elsif ($google_handler) {
+        	$google_ip = inet_ntoa(@{$google_handler->addr_list}[0]);
+	        &googlescan($google_ip, $target, $gdefined);
+        }
+}
         if($opt_w) {
 		header_reset();
                 &checkweb;
@@ -328,8 +375,8 @@ sub check_ip {
 
 sub banner {
 	log_print("###############################################################\n","c");
-	log_print("#      HackBot v$version 2002 / http://ws.obit.nl/hackbot/        #\n","c");
-	log_print("#               (c) 2000-2002 Marco van Berkum                #\n","c");
+	log_print("#      HackBot v$version 2003 / http://ws.obit.nl/hackbot/        #\n","c");
+	log_print("#               (c) 2000-2003 Marco van Berkum                #\n","c");
 	log_print("#                                                             #\n","c");
 	log_print("# Marco van Berkum            - m.v.berkum\@obit.nl            #\n","c");	
 	log_print("# Kristian Vlaardingerbroek   - kris\@obit.nl                  #\n","c");
@@ -353,7 +400,7 @@ sub cgi {
 	# actual code
 	log_print("Checking the webserver on port $port for various potential problems\n","h") unless($nodb);
 	log_print("------------------------------------------------------------------\n","h") unless($nodb);
-
+	
 	while(1) {
         	if($nodb) { 
 			log_print("No db found it should be in $database, skipping CGI scan\n","c");
@@ -361,13 +408,22 @@ sub cgi {
 		}	
 
 		if($proxy) {
-		print("Scanning via proxy $proxy\n\n");
+		log_print("Scanning via proxy $proxy\n\n","h");
                 while(<CGI>) {
 			if($opt_z) { sleep($opt_z); }
                         ($exploit, $description, $advisory) = split(/\!/, $_);
+
+# debug flag, where are we ?
+	log_print("checking for: $exploit\n","d");
+
                         chomp($exploit, $description, $advisory);
+
+
+
                         my $sock = new IO::Socket::INET (PeerAddr => $proxy, PeerPort => $proxyport, Proto => 'tcp', Timeout => '5');
                         if($sock && $exploit) {
+
+
                                 if($host_resolves == 1) {
                                         print $sock "GET http://$host$exploit HTTP/1.0\r\n\r\n";
                                 } else {
@@ -387,6 +443,10 @@ sub cgi {
 			if($opt_z) { sleep($opt_z); }
 			($exploit, $description, $advisory) = split(/\!/, $_);
 			chomp($exploit, $description, $advisory);
+
+# debug flag, where are we ?
+        log_print("checking for: $exploit\n","d");
+
 			my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => $port, Proto => 'tcp', Timeout => '5');
 			if($sock) {
 				if($host_resolves == 1) {
@@ -545,6 +605,7 @@ sub mta {
 
 sub checkweb {
 	undef($noweb);
+	log_print("\n","h");
         log_print("Checking for webserver on port $port\n","h");
 	log_print("-----------------------------------\n","h");
         my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => $port, Proto => 'tcp', Timeout => '5');
@@ -785,7 +846,7 @@ log_print("-----------------------\n","h");
                 log_print("huh, fscking TCP wrappers, I died\n\n","i");
                 last;
                 }
- 		undef($cnt);
+                undef($cnt);
                 log_print("FTP Server Found:\n$buff\n","c");
                 print $sock "pass bla\@bla.com\r\n";
                 sleep(2);
@@ -793,6 +854,7 @@ log_print("-----------------------\n","h");
                 if(!$cnt) {
                 last;
                 }
+
                 if ($buff =~ /230/) {
                 log_print("Anonymous login allowed\!\n\n","c");
                 log_print("Checking for upload/ directory\n\n","h");
@@ -1486,13 +1548,172 @@ sub xcheck {
 		sleep(3);
 		sysread($sock, $buff, 10000);
 			if($buff !~ /protocol/i && $buff !~ /autho/i) {
-				log_print("Allowed to connect!\n\n","c");
-			} else {
-			log_print("Connection not allowed\n\n","i");
+				log_print("Allowed to connect to X!\n\n","c");
+			} 
+		else
+		 {
+			log_print("Has X but connection is not allowed\n\n","c");
  		} 
+
 	}
+
 	if(!$sock) {
 	log_print("No X\n\n","i");
 	}
+
 }
 
+sub googlescan {
+
+        my $google_ip = $_[0];
+	my $target = $_[1];
+	my $gdefined = $_[2];
+
+        if (!-e $googledb) {
+                $nogoogle = 1;
+		log_print("Can't find google.db\n","c");
+                }
+
+if(!$gdefined) {
+
+	log_print("\n","h");
+	log_print("Google search\n","h") unless($nogoogle);
+	log_print("-------------\n","h") unless($nogoogle);
+
+	$stringcount = "0";
+
+        open STRINGS, "<$googledb" unless($nogoogle);
+
+	foreach(<STRINGS>) {	
+	
+	if($_ !~  /[a-z]/i) {
+		log_print("$googledb corrupted! skipping googlesearch\n\n","c");
+		$corrupted = "1";
+		last;	
+	}
+
+
+		$current = $_;
+		chomp($current);
+
+
+				my $sock = new IO::Socket::INET (PeerAddr => $google_ip, PeerPort => '80', Proto => 'tcp', Timeout => '5');
+
+   		if($sock) {
+
+
+		print $sock "GET http://www.google.com/search?as_q=inurl%3A$current&num=100&hl=en&ie=ISO-8859-1&btnG=Google+Search&as_epq=&as_oq=&as_eq=&lr=&as_ft=i&as_filetype=&as_qdr=all&as_occt=any&as_dt=i&as_sitesearch=$host&safe=images HTTP/1.0\r\n\r\n";
+		sleep(5);
+		@received  = <$sock>;
+
+		foreach(@received) {
+			$string = $_;
+			($temp, $string) = split(/a href=/, $string);
+                	if($string =~ /:\/\/$host/) {
+				($string, $temp) = split(/>/, $string);
+                                log_print("$string\n","c") unless($string !~ /$current/);
+                        }
+		}
+close $sock;
+	}
+	close STRINGS;
+	undef($current); undef($string); undef($sock); undef(@recieved);
+ }
+
+
+# google index search
+
+	my $sock = new IO::Socket::INET (PeerAddr => $google_ip, PeerPort => '80', Proto => 'tcp', Timeout => '5');
+	if($sock) {
+		print $sock "GET http://www.google.com/search?hl=en&lr=&ie=ISO-8859-1&q=%22Index+of+%2F%22+%22last+modified%22+site%3A$host HTTP/1.0\r\n\r\n";
+		sleep(5);	
+		@index  = <$sock>;
+			foreach(@index) { 
+			$index = $_;
+			($temp, $index) = split(/a href=/, $index);
+			if($index =~ /:\/\/$host/) {
+				($index, $temp) = split(/>/, $index);
+				log_print("$index - indexable directory\n","c"); 
+			}
+		}
+
+	log_print("\n","h");
+	undef($sock); undef($index); undef($temp);
+	}
+
+}
+
+
+if($gdefined) {
+
+	log_print("Google search for $gdefined \n","h") unless($nogoogle);
+	log_print("---------------------------------\n","h") unless($nogoogle);
+
+
+	my $sock = new IO::Socket::INET (PeerAddr => $google_ip, PeerPort => '80', Proto => 'tcp', Timeout => '5');
+	print $sock "GET http://www.google.com/search?as_q=inurl%3A$gdefined&num=100&hl=en&ie=ISO-8859-1&btnG=Google+Search&as_epq=&as_oq=&as_eq=&lr=&as_ft=i&as_filetype=&as_qdr=all&as_occt=any&as_dt=i&as_sitesearch=$host&safe=images HTTP/1.0\r\n\r\n";
+	sleep(5);
+	@received  = <$sock>;	
+		foreach(@received) {
+			$string = $_;
+                        ($temp, $string) = split(/a href=/, $string);
+                        if($string =~ /:\/\/$host/) {
+                                ($string, $temp) = split(/>/, $string);
+                                log_print("$string\n","c") unless($string !~ /$gdefined/);
+		}
+	}
+	}
+	log_print("\n\n","h");
+}
+
+
+sub pop3 {
+
+log_print("\nChecking for POP3 server\n","h");
+log_print("------------------------\n","h");
+        my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => 110, Proto => 'tcp', Timeout => '5');
+        if (!$sock) {
+        log_print("No POP3 server\n\n","i");
+        }
+	sleep(2);
+	        while($sock) {
+	                $cnt = sysread($sock, $buff, 10000);
+        	        if(!$cnt) {
+        	        log_print("huh, fscking TCP wrappers, I died\n\n","i");
+               		last;	
+                }
+        if($cnt) {
+	
+	log_print("$buff\n","c");
+	sleep(1);
+	close($sock);
+	undef($sock); undef($buff);
+	} }
+}
+
+sub evilsendmail {
+	$buggysendmailmsg = "* Possibly buggy sendmail\n\n\n";
+
+	my $sock = new IO::Socket::INET (PeerAddr => $target, PeerPort => 25, Proto => 'tcp', Timeout => '5');
+	if($sock) {
+		sleep(4);
+		sysread($sock, $buff, 1500);
+		if($buff =~ /Sendmail /) {
+			($temp, $buggyversion) 	= split(/Sendmail /,$buff);
+
+			$buggyversion =~ /(\d+?)\.(\d+?)\.(\d+?)/;
+
+			if (($1 && $2 && $3) &&
+			   (($1 < 8) ||
+			   (($1 == 8) && ($2 < 12)) ||
+			   (($1 == 8) && ($2 == 12) && ($3 < 9)))) {
+				print "$1 - $2 - $3\n";
+				log_print($buggysendmailmsg,"c");
+
+				undef($buggysendmailmsg); undef($buff); undef($temp); undef($buggyversion);
+				close($sock);
+				undef($sock);
+			}
+		}
+	}
+}
